"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RoomEncryptor = void 0;
exports.toRustHistoryVisibility = toRustHistoryVisibility;
var _matrixSdkCryptoWasm = require("@matrix-org/matrix-sdk-crypto-wasm");
var _event = require("../@types/event");
var _logger = require("../logger");
var _partials = require("../@types/partials");
/*
Copyright 2023 The Matrix.org Foundation C.I.C.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * RoomEncryptor: responsible for encrypting messages to a given room
 *
 * @internal
 */
class RoomEncryptor {
  /**
   * @param olmMachine - The rust-sdk's OlmMachine
   * @param keyClaimManager - Our KeyClaimManager, which manages the queue of one-time-key claim requests
   * @param outgoingRequestProcessor - The OutgoingRequestProcessor, which sends outgoing requests
   * @param room - The room we want to encrypt for
   * @param encryptionSettings - body of the m.room.encryption event currently in force in this room
   */
  constructor(olmMachine, keyClaimManager, outgoingRequestProcessor, room, encryptionSettings) {
    this.olmMachine = olmMachine;
    this.keyClaimManager = keyClaimManager;
    this.outgoingRequestProcessor = outgoingRequestProcessor;
    this.room = room;
    this.encryptionSettings = encryptionSettings;
    this.prefixedLogger = _logger.logger.getChild(`[${room.roomId} encryption]`);
  }

  /**
   * Handle a new `m.room.encryption` event in this room
   *
   * @param config - The content of the encryption event
   */
  onCryptoEvent(config) {
    if (JSON.stringify(this.encryptionSettings) != JSON.stringify(config)) {
      this.prefixedLogger.error(`Ignoring m.room.encryption event which requests a change of config`);
    }
  }

  /**
   * Handle a new `m.room.member` event in this room
   *
   * @param member - new membership state
   */
  onRoomMembership(member) {
    if (member.membership == "join" || member.membership == "invite" && this.room.shouldEncryptForInvitedMembers()) {
      // make sure we are tracking the deviceList for this user
      this.olmMachine.updateTrackedUsers([new _matrixSdkCryptoWasm.UserId(member.userId)]).catch(e => {
        this.prefixedLogger.error("Unable to update tracked users", e);
      });
    }

    // TODO: handle leaves (including our own)
  }

  /**
   * Prepare to encrypt events in this room.
   *
   * This ensures that we have a megolm session ready to use and that we have shared its key with all the devices
   * in the room.
   *
   * @param globalBlacklistUnverifiedDevices - When `true`, it will not send encrypted messages to unverified devices
   */
  async ensureEncryptionSession(globalBlacklistUnverifiedDevices) {
    var _this$room$getBlackli;
    if (this.encryptionSettings.algorithm !== "m.megolm.v1.aes-sha2") {
      throw new Error(`Cannot encrypt in ${this.room.roomId} for unsupported algorithm '${this.encryptionSettings.algorithm}'`);
    }
    const members = await this.room.getEncryptionTargetMembers();
    this.prefixedLogger.debug(`Encrypting for users (shouldEncryptForInvitedMembers: ${this.room.shouldEncryptForInvitedMembers()}):`, members.map(u => `${u.userId} (${u.membership})`));
    const userList = members.map(u => new _matrixSdkCryptoWasm.UserId(u.userId));
    await this.keyClaimManager.ensureSessionsForUsers(userList);
    this.prefixedLogger.debug("Sessions for users are ready; now sharing room key");
    const rustEncryptionSettings = new _matrixSdkCryptoWasm.EncryptionSettings();
    rustEncryptionSettings.historyVisibility = toRustHistoryVisibility(this.room.getHistoryVisibility());

    // We only support megolm
    rustEncryptionSettings.algorithm = _matrixSdkCryptoWasm.EncryptionAlgorithm.MegolmV1AesSha2;

    // We need to convert the rotation period from milliseconds to microseconds
    // See https://spec.matrix.org/v1.8/client-server-api/#mroomencryption and
    // https://matrix-org.github.io/matrix-rust-sdk-crypto-wasm/classes/EncryptionSettings.html#rotationPeriod
    if (typeof this.encryptionSettings.rotation_period_ms === "number") {
      rustEncryptionSettings.rotationPeriod = BigInt(this.encryptionSettings.rotation_period_ms * 1000);
    }
    if (typeof this.encryptionSettings.rotation_period_msgs === "number") {
      rustEncryptionSettings.rotationPeriodMessages = BigInt(this.encryptionSettings.rotation_period_msgs);
    }

    // When this.room.getBlacklistUnverifiedDevices() === null, the global settings should be used
    // See Room#getBlacklistUnverifiedDevices
    rustEncryptionSettings.onlyAllowTrustedDevices = (_this$room$getBlackli = this.room.getBlacklistUnverifiedDevices()) !== null && _this$room$getBlackli !== void 0 ? _this$room$getBlackli : globalBlacklistUnverifiedDevices;
    const shareMessages = await this.olmMachine.shareRoomKey(new _matrixSdkCryptoWasm.RoomId(this.room.roomId), userList, rustEncryptionSettings);
    if (shareMessages) {
      for (const m of shareMessages) {
        await this.outgoingRequestProcessor.makeOutgoingRequest(m);
      }
    }
  }

  /**
   * Discard any existing group session for this room
   */
  async forceDiscardSession() {
    const r = await this.olmMachine.invalidateGroupSession(new _matrixSdkCryptoWasm.RoomId(this.room.roomId));
    if (r) {
      this.prefixedLogger.info("Discarded existing group session");
    }
  }

  /**
   * Encrypt an event for this room
   *
   * This will ensure that we have a megolm session for this room, share it with the devices in the room, and
   * then encrypt the event using the session.
   *
   * @param event - Event to be encrypted.
   * @param globalBlacklistUnverifiedDevices - When `true`, it will not send encrypted messages to unverified devices
   */
  async encryptEvent(event, globalBlacklistUnverifiedDevices) {
    await this.ensureEncryptionSession(globalBlacklistUnverifiedDevices);
    const encryptedContent = await this.olmMachine.encryptRoomEvent(new _matrixSdkCryptoWasm.RoomId(this.room.roomId), event.getType(), JSON.stringify(event.getContent()));
    event.makeEncrypted(_event.EventType.RoomMessageEncrypted, JSON.parse(encryptedContent), this.olmMachine.identityKeys.curve25519.toBase64(), this.olmMachine.identityKeys.ed25519.toBase64());
  }
}

/**
 * Convert a HistoryVisibility to a RustHistoryVisibility
 * @param visibility - HistoryVisibility enum
 * @returns a RustHistoryVisibility enum
 */
exports.RoomEncryptor = RoomEncryptor;
function toRustHistoryVisibility(visibility) {
  switch (visibility) {
    case _partials.HistoryVisibility.Invited:
      return _matrixSdkCryptoWasm.HistoryVisibility.Invited;
    case _partials.HistoryVisibility.Joined:
      return _matrixSdkCryptoWasm.HistoryVisibility.Joined;
    case _partials.HistoryVisibility.Shared:
      return _matrixSdkCryptoWasm.HistoryVisibility.Shared;
    case _partials.HistoryVisibility.WorldReadable:
      return _matrixSdkCryptoWasm.HistoryVisibility.WorldReadable;
  }
}
//# sourceMappingURL=RoomEncryptor.js.map